#ifndef _RATING_H_
#define _RATING_H_


#include "board.h"
#include "pattern.h"
#include "mt19937p.h"


/* 戦術的特徴 */
enum FEATURE {
	F_SAVE_CAPTURE,		
	F_SAVE_EXTENSION,
	F_CAPTURE,	
	F_CAPTURE_AFTER_KO,	
	F_SELF_ATARI,
	F_ATARI,
	F_2POINT_SEMEAI,
	F_MAX,
};

#define TACTICAL_FEATURE F_MAX
#define BORDER_DISTANCE_MAX	11		// 10 : 19/2
#define MOVE_DISTANCE_MAX	55		// 54 : 18+18+18		


/*------------------------------------------------------------------*/
/* 構造体                                                           */
/*------------------------------------------------------------------*/

struct rate_set {
	float list[2];
};


/*-------------------------------------------------------------------*/
/* グローバル変数                                                    */
/*-------------------------------------------------------------------*/

/* γ */
extern float tactical_features[TACTICAL_FEATURE];
extern float border_distance[BORDER_DISTANCE_MAX];
extern float move_distance_1[MOVE_DISTANCE_MAX];
extern float move_distance_2[MOVE_DISTANCE_MAX];
extern float pattern3[PATTERN3_MAX];
extern float diamond[DIAMOND_MAX];

/* 盤端との距離のレート */
extern float border_rate[BOARD_MAX];

/* 初期値 */
extern struct rate_set init_rate_set[BOARD_MAX];


/*------------------------------------------------------------------*/
/* 関数                                                             */
/*------------------------------------------------------------------*/

/* 初期化 */
void InitializeRating( void );

/* 初期設定 */
void ClearRate( struct game *game );

/* 着手(Elo Rating) */
int RatingMove( struct game *game, int color, struct mt19937p *mt );

/* レーティング */
void Rating( struct game *game, int color );

/* レート計算 */
void UpdateRate( struct game *game, int pos, struct rate_set *rate_set );

/* 特徴の判定 */
void CheckFeatures( struct game *game, int color, struct rate_set *rate_set );

/* Capture after ko */
void CheckCaptureAfterKo( struct game *game, int color, struct rate_set *rate_set );

/* 自己アタリ */
void CheckSelfAtari( struct game *game, int color, int pos, struct rate_set *rate_set );

/* トリの判定 */
void CheckCapture( struct game *game, int color, int pos, struct rate_set *rate_set );

/* アタリの判定 */
void CheckAtari( struct game *game, int color, int pos, struct rate_set *rate_set );

/* γ読み込み */
void InputGamma( void );

/* テキスト入力 */
void InputTxtFLT( char *filename, float *ap, int array_size );

/* MoveDistance γ読み込み */
void InputMoveDistance( char *filename, float *ap );

/* Diamond Pattern γ読み込み */
void InputDiamond( char *filename );


#endif  /* _RATING_H_ */