#ifndef _UCT_H_
#define _UCT_H_


#include <windows.h>
#include <time.h>
#include "board.h"
#include "mt19937p.h"


/* スレッド数 */
#define THREAD_NUM 4

/* モード */
//#define UCT_MODE_1	// プレイアウト数を固定
#define UCT_MODE_2	// 思考時間を固定
//#define UCT_MODE_3	// 残り時間から、使用する時間を計算

#define CONST_PLAYOUT	10000	// 一定プレイアウト数
#define CONST_THINKING_TIME	0.9 //定思考時間

#define PLAYOUT_SPEED	2000	// 初期盤面のプレイアウト速度 PO/sec

#define ALL_THINKING_TIME	90		// 総思考時間
#define THINKING_TIME_RATE	14		// 残り時間に対して、使用する時間の割合 : 1/THINKING_TIME_RATE

#define PASS_RATE_1	0.98	// パス　1手前に相手がパス
#define PASS_RATE_2	0.95	// パス　パスしても勝率がこれ以上
#define RESIGN_RATE 0.99	// 投了

/* ノード */
#define MAX_NODE 1000000	// 最大ノード数
#define ROOT 0				// ルート

/* UCTパラメータ */
#define FPU 1.1				// 先頭打着緊急度
#define EQUIVALENCE	1000	// RAVEの等価パラメータ
#define PW	1.4				// Progressive Widening


/*------------------------------------------------------------------*/
/* 構造体                                                           */
/*------------------------------------------------------------------*/

/* スレッド関数の引数データ */
typedef struct _thread_arg {
	int thread_no;
	struct game *game;
	int color;
} thread_arg_t;

/* ノード */
struct node {
	int pos;
	int win;			// 勝数
	int move_count;		// 回数
	int rave_win;		// RAVEの勝数
	int rave_count;		// RAVEの回数
	int rave_sum;		// 子ノードのrave_countの合計
	int parent;			// 親ノード
	int parent_threshold;	// 親の閾値
	int child_origin;	// 子ノードの位置
	int child_num;		// 全子ノードの数
	int width;			// 探索幅
	float rate;			// レート
};

/* 着手回数 */
struct move_count {
	int colors[3];
};


/*------------------------------------------------------------------*/
/* 関数                                                             */
/*------------------------------------------------------------------*/

/* 初期化 */
void InitializeUCTMove( void );

/* 着手(UCT) */
int UCTMove( struct game *game, int color );

/*並列プレイアウト*/
void ParallelPlayOut( void *arg );

/* 1回プレイアウト */
void OnePlayOut( struct game *game, int color, struct mt19937p *mt );

/* ルートの展開 */
void RootExpantion( struct game *game, int color );

/* ノード展開 */
void NodeExpantion( struct game *game, int color, int index );

/* ノードのレーティング */
void RatingNode( struct game *game, int color, int index );

/* ノードをレートで比較する */
int CompareRate(const void* n1, const void* n2);

/* ノードを訪問回数で比較する */
int CompareMoveCount(const void *n1, const void *n2);

/* 最大のUCB値を持つ子ノードを返す */
int DecideByUCB( int index );

/* 更新 */
void UpdateData( int color, int index, int result, struct move_count *move_count );

/* Raveの回数を更新 */
void UpdateRaveCount( int color, int index, int result, struct move_count *move_count );

/* 候補手を表示 */
void DisplayCandidates( void );

/* 時間計測 */
double GetSpendTime(clock_t start_time);


#endif  /* _UCT_H_ */